import { useMemo, useSyncExternalStore } from "react";
export function useReactiveValue(value, ...args) {
    const mappedValue = useMemo(() => ({
        cache: null,
        cacheInvalidated: true,
        subscribe: (onValueChange) => {
            if (value) {
                return value.subscribe(() => {
                    mappedValue.cacheInvalidated = true;
                    return onValueChange();
                }, ...args);
            }
            return () => { };
        },
        get: () => {
            if (mappedValue.cacheInvalidated) {
                mappedValue.cache = value?.get(...args);
                mappedValue.cacheInvalidated = false;
            }
            return mappedValue.cache;
        },
    }), [value, ...args]);
    return useSyncExternalStore(mappedValue.subscribe, mappedValue.get, mappedValue.get);
}
//# sourceMappingURL=useReactiveValue.js.map