import { useCallback, useMemo, useRef, useSyncExternalStore } from "react";
export function useReactiveValues(value, onChange) {
    const onChangeRef = useRef(onChange);
    const valueChangeRef = useRef(() => { });
    const subscribeRef = useRef([]);
    onChangeRef.current = onChange;
    for (const unsubscribe of subscribeRef.current) {
        unsubscribe();
    }
    subscribeRef.current = [];
    const mappedValue = useMemo(() => ({
        cache: null,
        cacheInvalidated: true,
        subscribe: (onValueChange) => {
            valueChangeRef.current = onValueChange;
            return () => {
                for (const unsubscribe of subscribeRef.current) {
                    unsubscribe();
                }
            };
        },
        get: () => {
            if (mappedValue.cacheInvalidated) {
                mappedValue.cache = {};
                mappedValue.cacheInvalidated = false;
            }
            return mappedValue.cache;
        },
    }), [value]);
    useSyncExternalStore(mappedValue.subscribe, mappedValue.get, mappedValue.get);
    return useCallback((...args) => {
        if (value) {
            subscribeRef.current.push(value.subscribe(() => {
                mappedValue.cacheInvalidated = true;
                valueChangeRef.current();
                onChangeRef.current?.(...args);
            }, ...args));
            return value.get(...args);
        }
        return undefined;
    }, [value]);
}
//# sourceMappingURL=useReactiveValues.js.map